"""
GameStarter.py

A set of classes and functions common to most videogames.

by Dominic Pezzuto
"""

try:
    import pygame
    import os,sys
except:
    print "import error"

screenWidth = -1
screenHeight = -1    
screen = None
clock = pygame.time.Clock()
FPS = 60
keys = []

def init(screensize = (800,600), name = "My Game", fps = 60, fullscreen = False):
    """
    init(screensize = (800,600), name = "My Game")

    initialize the GameStarter module
    """
    global screenWidth, screenHeight, screen, FPS
    pygame.init()
    screenWidth, screenHeight = screensize
    if fullscreen:
        screen = pygame.display.set_mode((screenWidth, screenHeight), pygame.FULLSCREEN)
    else:
        screen = pygame.display.set_mode((screenWidth, screenHeight))
    pygame.display.set_caption(name)
    FPS = fps

def isPressed(char):
    global keys
    toCheck = getattr(pygame, "K_"+char)
    return keys[toCheck]

def eventHandler(event):
    """
    eventHandler(event)

    Empty function, hook an event handler function up to this.  Handles pygame events.
    """
    pass

def logic():
    """
    logic()

    Empty function, hook a function containing all of your logic up to this.
    """
    pass

def drawing():
    """
    drawing()

    Empty function, hook a function containing all of your drawing up to this
    """
    pass

def mainloop():    
    """
    mainloop()

    Checks for the quit event, then does the following:

    eventHandler(event)
    logic()
    drawing()
    pygame.display.flip()
    clock.tick(FPS)
    """
    global keys
    keys = pygame.key.get_pressed()
    
    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            sys.exit()
        else:
            eventHandler(event)
    logic()
    drawing()
    pygame.display.flip()
    clock.tick(FPS)
    
def getFPS():
    """
    getFPS():
    
    Get the current FPS of the show.
    """
    global FPS
    return FPS

def setFPS(newFPS):
    """
    setFPS(newFPS)

    Set a new FPS for this game.
    """
    global FPS
    FPS = newFPS

    
class Tile(pygame.sprite.Sprite):
    """
    Tile class, extends pygame.sprite.Sprite

    General class for tile objects, used for drawing backgrounds etc.
    """
    def __init__(self, image = None):
        super(Tile, self).__init__()
        self.image = image



class TileMap:
    """
    TileMap class

    A grouping class for Tile objects
    """
    
    def __init__(self, surface, tileWidth = -1, tileHeight = -1, vWrap = False, hWrap = False):
        ##super(TileMap,self).__init__()
        self.fullmap = []
        self.screenmap = []
        self.tiles = {}
        self.offset = [0,0]
        self.currentPosition = [1,0]
        self.surface = surface
        self.scrollSpeed = 2
        self.numRows = -1
        self.numCols = -1
        self.hWrap = hWrap
        self.vWrap = vWrap
        self.vCounter = 0
        self.tileWidth = tileWidth
        self.tileHeight = tileHeight
        if self.tileWidth != -1 and self.tileHeight != -1:
            self.numCols = self.surface.get_width() / self.tileWidth
            self.numRows = self.surface.get_height() / self.tileHeight
            

    def draw(self):
        #print len(self.screenmap[0])
##        print len(self.fullmap)
        if self.currentPosition[0] == 0 or len(self.fullmap[0])==1:
            resetx = 0
        else:
            resetx = -1 * self.tileWidth

        x = resetx
        
        if self.currentPosition[1] == 0 and not self.vWrap:
            y = 0
        else:
            y = -1 * self.tileHeight
        
        counter = 0
        
        for row in self.screenmap:            
            for position in row:
                if str(position) != "-1":                
                    imageToDraw = self.tiles[position].image                    
                    self.surface.blit( imageToDraw, ( (x + int(self.offset[0]) ), (y + int(self.offset[1]) ) ) )
                else:
                    imageToDraw = pygame.Surface((self.tileWidth, self.tileHeight))
                    
                counter += 1

                if counter != len( row ):                    
                    x += imageToDraw.get_width()
                else:
                    x = resetx
                    y += imageToDraw.get_height()
                    counter = 0
                


    def setMap(self, newmap):
        self.fullmap = newmap
        
        if self.tileWidth != -1 and self.tileHeight != -1:
            self.numCols = self.surface.get_width() / self.tileWidth
            self.numRows = self.surface.get_height() / self.tileHeight
            
        elif len(self.tiles) > 0:
            tile = self.tiles[self.tiles.keys()[0]]
            self.numCols = self.surface.get_width()/tile.image.get_width()
            self.numRows = self.surface.get_height()/tile.image.get_height()
            
        self.rebuildMap()
            

    def getMap(self):
        return self.fullmap

    def setTiles(self, newdic):
        self.tiles = newdic

    def getTiles(self):
        return self.tiles

    def addTiles(self, newtile):
            self.tiles[newtile[0]] = newtile[1]

    def scroll(self, dx, dy):        
        if dx>0:
            if self.currentPosition[0] > 0:
                if self.canScrollLeft() or (not self.canScrollLeft() and abs(self.offset[0])>=abs(dx)):
                    self.offset[0] += dx
            elif self.currentPosition[0] == 0 and abs(self.offset[0])>=abs(dx):
                self.offset[0] += dx
            elif self.hWrap:
                self.offset[0] = -999

        elif dx<0:
            if self.currentPosition[0] < (len(self.fullmap[0]) - self.numCols):
                if self.canScrollRight() or (not self.canScrollRight() and self.offset[0]>self.scrollSpeed):
                    self.offset[0] += dx
            elif (self.currentPosition[0] == (len(self.fullmap[0]) - self.numCols)) and self.offset[0]>self.scrollSpeed:
                self.offset[0] += dx
                                                                            
        if dy>0:
            if self.currentPosition[1] > 0 or self.vWrap:
                if self.canScrollUp() or (not self.canScrollUp() and abs(self.offset[1])>=abs(dy)):
                    self.offset[1] += dy
            elif (self.currentPosition[1] == 0 and abs(self.offset[1])>=abs(dy)) or self.vWrap:
                self.offset[1] += dy
        else:
            if self.currentPosition[1] < (len(self.fullmap) - self.numRows):
                if self.canScrollDown() or (not self.canScrollDown() and self.offset[1]>self.scrollSpeed):
                    self.offset[1] += dy
            elif ((self.currentPosition[1] == (len(self.fullmap) - self.numRows)) and self.offset[1]>self.scrollSpeed) or self.vWrap:
                self.offset[1] += dy
            
            
        doRebuild = False

        if self.offset[0] == -999:
            self.currentPosition[0] = len(self.fullmap[0])-self.numCols
            self.offset[0] = 0
            doRebuild = True
        
        if abs(self.offset[0]) > self.tileWidth:

            if self.offset[0] > 0:
                print "go"
                self.currentPosition[0] -= 1
            else:
                self.currentPosition[0] += 1

            self.offset[0] = 0
            doRebuild = True
                
        if abs(self.offset[1]) > self.tileHeight:

            if self.offset[1] > 0:
                self.currentPosition[1] -= 1
            else:
                self.currentPosition[1] += 1

            self.offset[1] = 0
            doRebuild = True

        if doRebuild:
            self.rebuildMap()


    def rebuildMap( self ):
        print self.currentPosition
        print len(self.fullmap) - self.numRows
        print "---"
        #self.screenmap = self.fullmap
        self.screenmap = []

        if abs(self.currentPosition[1]) == len(self.fullmap):
            self.currentPosition[1] = 0
            self.vCounter = 0

        counter = [0,0]        
        
        if self.currentPosition[1]>0:
            startrow = self.currentPosition[1]-1
            ylookahead = 2
        elif not self.vWrap: #currentPos[1] = 0
            startrow = 0
            ylookahead = 1
        else:
            startrow = 1
            ylookahead = self.vCounter

        if self.currentPosition[0]>0:
            startcol = self.currentPosition[0]-1
            xlookahead = 2
        elif not self.hWrap: #currentPos[0] = 0
            startcol = 0
            xlookahead = 1
        else:
            startrow = 0
            ylookahead = 1
            
##        self.screenmap = self.fullmap
##        #Get each row (vWrap)
##        self.screenmap = self.screenmap[startrow: startrow+self.numRows+ylookahead ]
##
##        if self.vWrap:
##            if self.currentPosition[1] > 0:
##                counter[1] = 0
##                while len(self.screenmap) < self.numRows+2:
##                    self.screenmap += [self.fullmap[counter[1]]]
##                    counter[1] += 1
##            elif self.currentPosition[1] <= 0:
##                counter[1] = -1
##                while len(self.screenmap) < self.numRows+2:
##                    print "oop"
##                    self.screenmap = [self.fullmap[counter[1]]] + self.screenmap
##                    counter[1] -= 1
##                self.vCounter -= 1
        
            
        if self.vWrap or (self.currentPosition[1]!=0 and self.currentPosition[1]!=len(self.fullmap)-self.numRows):
##            tcount = -2
            tcount = -1
            pos = self.currentPosition[1]-1
        elif self.currentPosition[1]==len(self.fullmap)-self.numRows:
            tcount = -1
            pos = self.currentPosition[1]-1
        else:
            tcount = -1
            pos = self.currentPosition[1]
            


        while tcount < self.numRows:
            if pos > len(self.fullmap)-1:
                pos=0
            self.screenmap += [ self.fullmap[pos] ]
            pos += 1
            tcount += 1
            
                

        tmp = []        

        for row in self.screenmap:
            tmp += [row[startcol: startcol+self.numCols+xlookahead]]
        
        self.screenmap = tmp
        
        for row in self.screenmap:
            print str(row)
        print "**"


    def canScrollRight( self ):
        for row in self.screenmap:
#            if not(row == self.screenmap[0] and self.currentPosition[1]>0) or (row == self.screenmap[0] and self.currentPosition[1]==0):            
            if (((not(row == self.screenmap[0] and self.currentPosition[1]>0 )) or (row == self.screenmap[0] and self.currentPosition[1]==0) or (self.currentPosition[1]>0 and self.offset[1]>=0 and row == self.screenmap[0])) \
               and (not(row == self.screenmap[-1] and self.currentPosition[1]< len(self.fullmap)-self.numRows )\
                    or (row == self.screenmap[-1] and self.currentPosition[1]== len(self.fullmap)-self.numRows )\
                    or (row == self.screenmap[-1] and self.currentPosition[1] < len(self.fullmap)-self.numRows and self.offset[1]<0))\
                    or (self.vWrap and self.screenmap[0]!=row and self.screenmap[-1]!=row)):
                if str(row[len(row)-1]) == "-1":
                    return False
        return True
    
    def canScrollLeft( self ):
        for row in self.screenmap:
            if (not(row == self.screenmap[0] and self.currentPosition[1]>0) or (row == self.screenmap[0] and self.currentPosition[1]==0) or (self.currentPosition[1]>0 and self.offset[1]>=0 and row == self.screenmap[0])) \
               and (not(row == self.screenmap[-1] and self.currentPosition[1]< len(self.fullmap)-self.numRows )\
                    or (row == self.screenmap[-1] and self.currentPosition[1]== len(self.fullmap)-self.numRows )\
                    or (row == self.screenmap[-1] and self.currentPosition[1] < len(self.fullmap)-self.numRows and self.offset[1]<0)):            
                if str(row[0]) == "-1":
                    return False
        return True    

    def canScrollUp( self ):
        for position in self.screenmap[0]:
##            if ((not(self.currentPosition[0]==0 or position==self.screenmap[0][0])) or self.currentPosition[0]==0 or (self.currentPosition[0]>0 and position== self.screenmap[0][0] and self.offset[0]>0))\
##            and ((self.currentPosition[0]!=len(self.fullmap[0])-self.numCols and position != self.screenmap[0][-1])\
##                 or self.currentPosition[0] == len(self.fullmap[0])-self.numCols or (self.currentPosition[0]<len(self.fullmap[0])-self.numCols and position== self.screenmap[0][-1] and self.offset[0]<0)):
            if ((not(self.currentPosition[0]==0 or position==self.screenmap[0][0])) or self.currentPosition[0]==0 or (self.currentPosition[0]>0 and position== self.screenmap[0][0] and self.offset[0]>0))\
            and ((self.currentPosition[0]!=len(self.fullmap[0])-self.numCols and position != self.screenmap[0][-1])\
                 or self.currentPosition[0] == len(self.fullmap[0])-self.numCols or (self.currentPosition[0]<len(self.fullmap[0])-self.numCols and position== self.screenmap[0][-1] and self.offset[0]<0))\
                 or (self.vWrap and (self.screenmap[0][0]!=position and self.screenmap[0][-1]!=position)):
                if str(position) == "-1":
                    return False
        return True

    def canScrollDown( self ):
        for position in self.screenmap[-1]:
            if (((self.currentPosition[0]!=0 and position!=self.screenmap[-1][0]) or self.currentPosition[0]==0 or (self.currentPosition[0]>0 and position== self.screenmap[-1][0] and self.offset[0]>0))\
            and ((self.currentPosition[0]!=len(self.fullmap[0])-self.numCols and position != self.screenmap[-1][-1])\
                 or self.currentPosition[0] == len(self.fullmap[0])-self.numCols or (self.currentPosition[0]<len(self.fullmap[0])-self.numCols and position== self.screenmap[-1][-1] and self.offset[0]<0)))\
                 or (self.vWrap and (self.screenmap[-1][0]!=position and self.screenmap[-1][-1]!=position)):
                if str(position) == "-1":
                    return False
        return True                


class Sprite ( pygame.sprite.Sprite ):
    def __init__(self, image = None):
        super(Sprite, self).__init__()
        if image!= None:
            self.image = image.copy()
        else:
            self.image = pygame.Surface((0,0))
        self.rect = pygame.Rect(0.0,0.0,10.0,10.0)
        self.frames = {}
        self.currentFrame = 0
        self.frameCounter = 0
        self.frameChange = 20
        self.moveSpeed = [2,2]
        self.updateRect()
        self.currentFrameset = ""

    def updateRect(self):
        if self.image != None:
            self.rect = pygame.Rect(self.rect.left, self.rect.top,
                                    self.image.get_width(), self.image.get_height())
    def update ( self ):
        self.animate( self.currentFrameset )
        self.rect = pygame.Rect( [self.rect[0], self.rect[1], self.image.get_width(), self.image.get_height() ] )        

    def move ( self, dx, dy ):
        self.rect.move_ip( dx, dy )

    def moveSpaces( self, dx, dy ):
##        self.rect[0] = float(self.rect[0]) + (float(dx) * float(self.moveSpeed[0]))        
##        self.rect[1] = float(float(self.rect[1]) + (float(dy) * float(self.moveSpeed[1])))
        self.rect.move_ip( float(dx) * float(self.moveSpeed[0]), float(dy) * float(self.moveSpeed[1]) )
    def setPos(self, pos):
        self.rect[0] = float(pos[0])
        self.rect[1] = float(pos[1])

    def setFrameset (self, newFS):
        self.currentFrameset = newFS

    def animate(self, frameset):
        try:
            frameToDraw = self.frames[frameset][self.currentFrame]
            #surface.blit( frameToDraw, (self.rect[0], self.rect[1]) )
            self.image = frameToDraw
            
            if self.frameCounter >= self.frameChange:
                self.frameCounter = 0            
                if self.currentFrame >= len( self.frames[frameset] )-1:
                    self.currentFrame = 0
                else:
                    self.currentFrame += 1

            self.frameCounter += 1
        except:
            pass
    
    def draw(self, surface):
        surface.blit(self.image, self.rect)#(self.rect[0], self.rect[1]) )


class ActiveSprites( pygame.sprite.Group ):
    def __init__( self ):
        super(ActiveSprites, self).__init__()
        
